/* ------------------------------------------------------------------ 

	Title: WGGraphAxis

	Description:  A class to be used in constructing graph axis.
		It provides base operations for the abscissa & ordinate.

	Author :
		Stephen Wardlaw, M.D.
		Yale University School of Medicine
		20 York St.
		New Haven, CT  06504

	Edit History:
		09 Dec 97
			Adapted for BeOS
		09 Feb 94
			Split again from 'Abscissa' & 'Ordinate'
		24 Jan 94
			Split from the old Module Graph in MW M-2 and
			re-written for MetroWerks C++
		16 May 90
			Modified for MetroWerks Modula-2 on Macintosh
		06 Nov 87
			Last IBM Version 6.0
		?? Aug 80
			First version for Apple IIe
		
------------------------------------------------------------------ */ 

#include "WGGraphAxis.h"
#include "UFloating_Util.h"

// -------------------------------------------------------------------
// *  Constructors & Destructor             
// -------------------------------------------------------------------
WGGraphAxis::WGGraphAxis()
{
	mAxis.max = 100.0;
	mAxis.min = 0.0;
	mAxis.ticInterval = 10.0;
	mAxis.minorDivs = 2;
	mAxis.scale = scale_linear;
	mAxis.numFmt = realFmt_fixed;
	mAxis.numDPs = 0;
	mTicsLabeled = true;
	mExtFunct = NULL;
}
// -------------------------------------------------------------------
// *  Public Methods             
// -------------------------------------------------------------------
// This method sets the axis and checks for errors. If errors are found,
//	it corrects them and returns false.
bool
WGGraphAxis::SetAxis(SAxis &newAxis)
{
	bool valid = true;
	
	// Change title and test bounds
	if(newAxis.scale == scale_log) {
		if(newAxis.min <= 0.0) {
			// Axis Minimum Must Be > 0 With Log Scale
			newAxis.min = 1;
			if(newAxis.max <= newAxis.min) {
				newAxis.max = newAxis.min + 1.0;
			}
			valid = false;
		}
		if(newAxis.ticInterval < 1.10) {
			newAxis.ticInterval = 1.10;
			// Log Base Below Minimum
			valid = false;
		}
		
		if(newAxis.ticInterval > 20.0) {
			newAxis.ticInterval = 20.0;
			//Log Base Over Maximum
			valid = false;
		}
	}	
	
	// Ensure max > min
	if(newAxis.max <= newAxis.min) {
		newAxis.min = newAxis.max + 1.0;
		// Max Must Be > Min"
		valid = false;
	}
	// Check for too small a tic interval
	if(!(newAxis.scale == scale_log || (newAxis.ticInterval > 0.0 && (newAxis.max - newAxis.min)/newAxis.ticInterval <= 20.0))) {
		// Tic Interval Too Small");
		newAxis.ticInterval = (newAxis.max - newAxis.min)/20.0;
		valid = false;
	}
	// Set new values if all is ok
	if(valid) {
		mAxis = newAxis;
	}
	
	return valid;
}
// -------------------------------------------------------------------
// *  Protected Methods             
// -------------------------------------------------------------------
// Transforms numbers from graph coord #s to orig input values
double
WGGraphAxis::XformFrom(double num) 
{
	switch (mAxis.scale) {
	
		case scale_linear:
			return num;
		
		case scale_log:
			if(num < log(HUGE_VAL)) {
				return exp(num);
			} else {
				return HUGE_VAL;
			}
		case scale_funct:
			if(mExtFunct != 0) {
				return mExtFunct->XformFrom(num);
			} else {
				return num;
			}
		default:
			return num;
	}
}	
// -------------------------------------------------------------------
// Transforms numbers from input values to the graph coord #s
double
WGGraphAxis::XformTo(double num) 
{
	switch (mAxis.scale) {
	
		case scale_linear:
			return num;
		
		case scale_log:
			if(num > 0.0) {
				return log(num);
			} else {
				return min_num;
			}
		case scale_funct:
			if(mExtFunct != 0) {
				return mExtFunct->XformTo(num);
			} else {
				return num;
			}
		default:
			return num;
	}
}	
