/* ------------------------------------------------------------------ 

	Title: WGGraphAxis

	Description:  A class to be used in constructing graph axis.
		It provides base operations for the abscissa & ordinate.

	Author :
		Stephen Wardlaw, M.D.
		Yale University School of Medicine
		20 York St.
		New Haven, CT  06504

	Edit History:
		09 Dec 97
			Adapted for BeOS
		09 Feb 94
			Split again from 'Abscissa' & 'Ordinate'
		24 Jan 94
			Split from the old Module Graph in MW M-2 and
			re-written for MetroWerks C++
		16 May 90
			Modified for MetroWerks Modula-2 on Macintosh
		06 Nov 87
			Last IBM Version 6.0
		?? Aug 80
			First version for Apple IIe
		
------------------------------------------------------------------ */ 

#pragma once

#include "WMFunctBase.h"
#include "URealFmt_Utils.h"

const float		border_wd	= 2;	// Width for drawing border & major tics
const double	min_num		= 1E-10;// Minimum allowable number
const float		tic_len		= 9;	// Length of a graph tic
const float 	tic_space	= 3;	// Space between tic and character

enum EAxisScale {
	scale_linear,
	scale_log,
	scale_funct
	};
	
struct SAxis {
	short	double	max,min;			// Axis limits
	short double	ticInterval;	// Interval for labelling & major tics
	short				minorDivs;		// Minor axis divisions
	EAxisScale		scale;			// Scale type
	ERealFormat		numFmt;			// Axis marking format
	short				numDPs;			// # of DPs/SigFigs
	};
    
class WGGraphAxis : public WMFunctBase {

	public:
	
		WGGraphAxis();
		
	void				// Gets the current axis settings
		GetAxis(SAxis &theAxis) {theAxis = mAxis;}
		
	void				// Gets the user-defined plotting function, if any
		GetExtFunction(WMFunctBase* &functPtr) {functPtr = mExtFunct;}
		
	bool				// Sets new axis parameters
		SetAxis(SAxis &newAxis);
		
	void				// Sets a user-defined axis transfer function
		SetExtFunction(WMFunctBase* newFunct);
		
	void
		SetTickLabeling(bool labelTics) {mTicsLabeled = labelTics;}
		
	virtual double	// Axis->data transform
		XformFrom(double num);
		
	virtual double	// Data->axis transform
		XformTo(double num);
	
	public:	// Data
	
	SAxis			mAxis;				// The public axis record
	WMFunctBase*mExtFunct;			// Client-supplied transforming functions, if any
	BFont			mAxisFont;			// The current font
	float			mFontHeight;		// The working font height
	bool			mTicsLabeled;		// Are the major tics labeled?
	};
    
